/*
 *   IRC - Internet Relay Chat, src/modules/m_uline.c
 *   (C) 2005 Tiburon <tiburon@gmail.com>
 *
 *   See file AUTHORS in IRC package for additional names of
 *   the programmers. 
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "config.h"
#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include <time.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifdef _WIN32
#include <io.h>
#endif
#include <fcntl.h>
#include "h.h"
#include "proto.h"
#ifdef STRIPBADWORDS
#include "badwords.h"
#endif
#ifdef _WIN32
#include "version.h"
#endif

#define MSG_ULINE 	"ULINE"
#define TOK_ULINE 	"UL"

DLLFUNC int m_uline(aClient *cptr, aClient *sptr, int parc, char *parv[]);
#ifndef DYNAMIC_LINKING
ModuleHeader m_uline_Header
#define Mod_Header m_uline_Header
#else
ModuleHeader Mod_Header
#endif
  = {
	"uline",	/* Name of module */
	"$Id: m_uline.c,v 1.1.1.1 2009/01/03 21:01:10 tirc Exp $", /* Version */
	"/uline", /* Short description of module */
	"3.2-b8-1",
    };

/*
 * The purpose of these ifdefs, are that we can "static" link the ircd if we
 * want to
*/

#ifdef DYNAMIC_LINKING
DLLFUNC int	Mod_Init(ModuleInfo *modinfo)
#else
int    m_uline_Init(ModuleInfo *modinfo)
#endif
{
	/*
	 * We call our add_Command crap here
	*/
	add_Command(MSG_ULINE, TOK_ULINE, m_uline, MAXPARA);
	return MOD_SUCCESS;
	
}
#ifdef DYNAMIC_LINKING
DLLFUNC int	Mod_Load(int module_load)
#else
int    m_uline_Load(int module_load)
#endif
{
	return MOD_SUCCESS;
	
}
#ifdef DYNAMIC_LINKING
DLLFUNC int	Mod_Unload(int module_unload)
#else
int	m_uline_Unload(int module_unload)
#endif
{
	if (del_Command(MSG_ULINE, TOK_ULINE, m_uline) < 0)
	{
		sendto_realops("Failed to delete commands when unloading %s",
				Mod_Header.name);
	}
	return MOD_SUCCESS;
	
}

/* 
 * m_uline - 1/16/2003 - Tiburon
 * :prefix ULINE <nick>
 * parv[0] - sender
 * parv[1] - nickname
 *
*/

DLLFUNC int m_uline(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
	aClient *acptr;

	if (MyClient(sptr))
		if (!IsNetAdmin(sptr))
		{
			if (!IsOper(sptr) || !IsULine(sptr))
			{
			sendto_one(sptr, err_str(ERR_NOPRIVILEGES), me.name,
			    parv[0]);
			return 0;
			}
		}
	if (parc < 2)
	{
		sendto_one(sptr,
		    ":%s NOTICE %s : /uline syntax is /uline <nick>",
		    me.name, sptr->name);
		return 0;
	}

	if (strlen(parv[1]) < 1)
	{
		sendto_one(sptr,
		    ":%s NOTICE %s : You are missing the nick",
		    me.name, sptr->name);
		return 0;
	}

	if (strlen(parv[1]) > (NICKLEN))
	{
		sendto_one(sptr,
		    ":%s NOTICE %s : uline Error: Nickname too long",
		    me.name, sptr->name);
		return 0;
	}
	/* OK, at this point, the pass is valid */
	if ((acptr = find_person(parv[1], NULL)))
	{
			if IsULine(acptr)
			{
				sendto_snomask(SNO_EYES, "%s disabled %s's U:Line", sptr->name, acptr->name);
				acptr->flags &= ~FLAGS_ULINE;
			if (MyConnect(acptr))
                        sendto_one(acptr,
                                ":%s %s %s : You are no longer U:Lined",
                                me.name, IsWebTV(acptr) ? "PRIVMSG" : "NOTICE",
                                acptr->name);
                	if (MyConnect(sptr) && acptr != sptr)
                        sendto_one(sptr,
                                ":%s %s %s : %s is no longer U:Lined",
                                me.name, IsWebTV(acptr) ? "PRIVMSG" : "NOTICE",
                                sptr->name, acptr->name);
                        }
			else
			{
				sendto_snomask(SNO_EYES, "%s made %s a U:Line", sptr->name, acptr->name);
				acptr->flags |= FLAGS_ULINE;
		if (MyConnect(acptr))
			sendto_one(acptr,
				":%s %s %s : You are now U:Lined",
				me.name, IsWebTV(acptr) ? "PRIVMSG" : "NOTICE",
				acptr->name);
		if (MyConnect(sptr) && acptr != sptr)
			sendto_one(sptr,
				":%s %s %s : %s is now U:Lined",
				me.name, IsWebTV(acptr) ? "PRIVMSG" : "NOTICE",
				sptr->name, acptr->name);
			}
		sendto_serv_butone_token(cptr, sptr->name,
		    MSG_ULINE, TOK_ULINE, "%s", acptr->name);
	}
	else
	{
		sendto_one(sptr, err_str(ERR_NOSUCHNICK), me.name, sptr->name,
		    parv[1]);
		return 0;
	}
	return 0;
}

